from param import *

"""
Ce fichier définit plusieurs fonctions permettant d'avoir des renseignements
sur le monde
"""
def est_vide(monde, ligne, colonne):
    """
    Fonction permettant de savoir si la case (ligne, colonne) du monde
    est vide
    """
    return monde[ligne][colonne] is None

def est_mouton(monde, ligne, colonne):
    """
    Fonction permettant de savoir si la case (ligne, colonne) du monde
    contient un mouton
    """
    return monde[ligne][colonne] is not None and monde[ligne][colonne][INDICE_TYPE] == TYPE_MOUTON

def est_loup(monde, ligne, colonne):
    """
    Fonction permettant de savoir si la case (ligne, colonne) du monde
    contient un loup
    """
    return monde[ligne][colonne] is not None and monde[ligne][colonne][INDICE_TYPE] == TYPE_LOUP

def case_valide(ligne, colonne):
    """
    Fonction qui détermine si des un couple de coordonnées (ligne, colonne)
    est valide (correspond à une case de la grille) ou pas
    """
    if ligne is None or colonne is None:
        return False
    if ligne < 0 or ligne >= TAILLE:
        return False
    if colonne < 0 or colonne >= TAILLE:
        return False
    return True

def case_a_gauche(ligne, colonne):
    """
    Fonction renvoyant les coordonnées de la case à gauche de la case courante.
    Renvoie None, None si cette case n'existe pas (car nous sommes déjà sur le bord gauche)
    """
    if colonne > 0:
        return ligne, colonne - 1
    return None, None

def case_a_droite(ligne, colonne):
    """
    Fonction renvoyant les coordonnées de la case à droite de la case courante.
    Renvoie None, None si cette case n'existe pas (car nous sommes déjà sur le bord droit)
    """
    if colonne < TAILLE - 1:
        return ligne, colonne + 1
    return None, None

def case_au_dessus(ligne, colonne):
    """
    Fonction renvoyant les coordonnées de la case au dessus de la case courante.
    Renvoie None, None si cette case n'existe pas (car nous sommes déjà sur le bord supérieur)
    """
    if ligne > 0:
        return ligne - 1, colonne
    return None, None

def case_au_dessous(ligne, colonne):
    """
    Fonction renvoyant les coordonnées de la case au dessous de la case courante.
    Renvoie None, None si cette case n'existe pas (car nous sommes déjà sur le bord inférieur)
    """
    if ligne < TAILLE - 1:
        return ligne + 1, colonne
    return None, None

def cases_vers(ligne_actuelle, colonne_actuelle, ligne_cible, colonne_cible):
    """
    Fonction qui détermine les éventuelles 2 cases voisines de la case actuelle qui
    permettent de se rapporcher d'une case cible. Cette fonction ne vérifie pas si
    les cases en question sont vides.
    """
    if ligne_actuelle == ligne_cible and colonne_actuelle == colonne_cible:
        return []
    #assert ligne_actuelle != ligne_cible or colonne_actuelle != colonne_cible, "pb : " + str(ligne_actuelle) + ","  + str(colonne_actuelle) + " = " + str(ligne_cible) + "," + str(colonne_cible)
    #print("déplacement de",ligne_actuelle,",",colonne_actuelle,"->",ligne_cible,",",colonne_cible)
    destinations = []
    delta_ligne = ligne_cible - ligne_actuelle
    delta_colonne = colonne_cible - colonne_actuelle
    if delta_ligne > 0:
        nouvelle_ligne = ligne_actuelle + 1
    elif delta_ligne < 0:
        nouvelle_ligne = ligne_actuelle - 1
    else:
        nouvelle_ligne = ligne_actuelle
    if delta_colonne > 0:
        nouvelle_colonne = colonne_actuelle + 1
    elif delta_ligne < 0:
        nouvelle_colonne = colonne_actuelle - 1
    else:
        nouvelle_colonne = colonne_actuelle
    if case_valide(nouvelle_ligne, colonne_actuelle) and nouvelle_ligne != ligne_actuelle:
        destinations.append((nouvelle_ligne, colonne_actuelle))
    if case_valide(ligne_actuelle, nouvelle_colonne) and nouvelle_colonne != colonne_actuelle:
        destinations.append((ligne_actuelle, nouvelle_colonne))
    return destinations

def mouton_le_plus_proche(ligne, colonne, liste_moutons):
    """
    Fonction qui cherche le mouton le plus proche d'une case donnée.
    """
    objectif = None
    distance_min = TAILLE*2
    for mouton in liste_moutons:
        distance = abs(mouton[INDICE_LIGNE] - ligne) + abs(mouton[INDICE_COLONNE] - colonne)
        if distance < distance_min:
            distance_min = distance
            objectif = mouton
    return objectif

def loups_les_plus_proches(ligne, colonne, nb_loups, liste_loups):
    """
    Fonction qui recherche les nb_loups les plus proches de la
    case (ligne, colonne).
    S'il n'y a pas assez de loups, la fonction renvoie la liste
    de tous les loups.
    Dans la liste renvoyés, les loups sont triés du plus près au plus loin;
    """
    loups_tries = sorted(liste_loups, key = lambda loup: abs(loup[INDICE_LIGNE] - ligne) + abs(loup[INDICE_COLONNE] - colonne))
    return loups_tries[:nb_loups]

def voisins_libres(monde, ligne, colonne):
    """
    Fonction renvoyant la liste des cases libres voisines de
    la case (ligne, colonne)
    """
    retour = []
    if ligne > 0 and est_vide(monde, ligne - 1, colonne):
        retour.append((ligne - 1, colonne))
    if ligne < TAILLE - 1 and est_vide(monde, ligne + 1, colonne):
        retour.append((ligne + 1, colonne))
    if colonne > 0 and est_vide(monde, ligne, colonne - 1):
        retour.append((ligne, colonne - 1))
    if colonne < TAILLE - 1 and est_vide(monde, ligne, colonne + 1):
        retour.append((ligne, colonne + 1))
    return retour

def moutons_voisins(monde, ligne, colonne):
    """
    Fonction renvoyant la liste des moutons voisins de la case
    de coordonnées (ligne, colonne)
    """
    moutons = []
    if ligne > 0 and est_mouton(monde, ligne - 1, colonne):
        moutons.append(monde[ligne - 1][colonne])
    if ligne < TAILLE - 1 and est_mouton(monde, ligne + 1, colonne):
        moutons.append(monde[ligne + 1][colonne])
    if colonne > 0 and est_mouton(monde, ligne, colonne - 1):
        moutons.append(monde[ligne][colonne - 1])
    if colonne < TAILLE - 1 and est_mouton(monde, ligne, colonne + 1):
        moutons.append(monde[ligne][colonne + 1])
    return moutons
